#include <avr/io.h>
#include <avr/interrupt.h>
#include "io_label.h"
#include "sbit.h"
#include "led_wuerfel.h"
#include "timer.h"

#define min_prescaler_grenze_verschieben 0
#define max_prescaler_grenze_verschieben 2000
#define auslaufen_prescaler 4


void init_timer0(void) {
	TCCR0 |= (1<<CS01);	//Prescaler 8
	TIMSK |= (1<<TOIE0);	//Interrupt bei Timer Overflow
	sei();					//Interrupts aktivieren
}




int8_t count_g = -1;						//aktueller Zhlwert
int8_t key_g_pressed;						//Taste wurde seit dem letzten Auslaufen gedrckt
int8_t key_g_last = true;					//Wert der Taste im letzten Zyklus; fr Flankenerkennung
int8_t key_g_abbild;						//Abbild des Tasters, damit wrend des gesamten Interrupt-Codes auf gleichen Datenbestand zugegriffen wird
uint16_t prescaler_pressed_led_grenze_g;	
uint16_t prescaler_pressed_led_g;
uint16_t prescaler_grenze_verschieben_g;

int8_t count_r = -1;						//aktueller Zhlwert
int8_t key_r_pressed;						//Taste wurde seit dem letzten Auslaufen gedrckt
int8_t key_r_last = true;					//Wert der Taste im letzten Zyklus; fr Flankenerkennung
int8_t key_r_abbild;						//Abbild des Tasters, damit wrend des gesamten Interrupt-Codes auf gleichen Datenbestand zugegriffen wird
uint16_t prescaler_pressed_led_grenze_r;
uint16_t prescaler_pressed_led_r;
uint16_t prescaler_grenze_verschieben_r;

ISR(TIMER0_OVF_vect) {
		key_g_abbild = KEY_G;
		if (key_g_abbild != key_g_last && key_g_abbild == true) { //fallende Flanke -> gedrckt, da KEY_G invertiert
					key_g_pressed = false; //auf Standartwerte setzen, damit durch dieses Bettigen alles wie immer ist
					prescaler_pressed_led_grenze_g = min_prescaler_grenze_verschieben;
					prescaler_pressed_led_g = 0;
					prescaler_grenze_verschieben_g = 0;
		}
	
		if (key_g_abbild == false) {	//wenn Taster gedrckt wird, dann
			set_wuerfel_g(-1);	//alle Segmente zum leuchten bringen
			if (count_g == 6) { //von 1-6 zhlen
				count_g = 1;
			} else {
				count_g++;
			}	
		}	
	
		if (key_g_abbild != key_g_last && key_g_abbild == true) { //steigende Flanke -> losgelassen, da KEY_G invertiert
			key_g_pressed = true;
		}
		
		if (key_g_pressed == true) {										//wenn taste gedrckt, aber inzwischen losgelassen wurde, dann
			if(prescaler_pressed_led_g < prescaler_pressed_led_grenze_g) {	//prescaler - zhlen von 0 bis zur dynamischen prescalerobergrenze (das dynamische wird fr das auslaufen bentigt)
				prescaler_pressed_led_g++;
			} else {
				prescaler_pressed_led_g = 0;
			}
			if (prescaler_grenze_verschieben_g < auslaufen_prescaler) { //prescaler fr die geschwindigkeit des auslaufens -> ruft bei 0 die erhhung des dynamischen prescalers auf
				prescaler_grenze_verschieben_g++;
			} else {
				prescaler_grenze_verschieben_g = 0;
			}
			
			
			
			
			if (prescaler_pressed_led_g == 0) { //wenn overflow des dynamischen prescalers, dann
				if (count_g == 6) { //von 0 bis 6 zhlen
					count_g = 1;
				} else {
					count_g++;
				}
				set_wuerfel_g(count_g);	//und ausgeben
			}			
			
			if (prescaler_grenze_verschieben_g == 0) { //wenn overflow des prescalers fr das verschieben der geschwindigkeit, dann
				if (prescaler_pressed_led_grenze_g < max_prescaler_grenze_verschieben) { //wenn langsamste geschwindigkeit noch nicht erreicht, dann
					prescaler_pressed_led_grenze_g++; //geschwindigkeit verlangsamen (durch erhhung des prescalers)
				} else {
					key_g_pressed = false; //sonst alles auf standartwerte setzen und das auslaufen verlassen (zahl bleib stehen!)
					prescaler_pressed_led_grenze_g = min_prescaler_grenze_verschieben;
					prescaler_pressed_led_g = 0;
					prescaler_grenze_verschieben_g = 0;
				}
			}
		}
		
		key_g_last = key_g_abbild;
		
		
		
		
		
		
		
		
		
		
		
		key_r_abbild = KEY_R;
		if (key_r_abbild != key_r_last && key_r_abbild == true) { //fallende Flanke -> gedrckt, da KEY_G invertiert
			key_r_pressed = false; //auf Standartwerte setzen, damit durch dieses Bettigen alles wie immer ist
			prescaler_pressed_led_grenze_r = min_prescaler_grenze_verschieben;
			prescaler_pressed_led_r = 0;
			prescaler_grenze_verschieben_r = 0;
		}
				
		if (key_r_abbild == false) {	//wenn Taster gedrckt wird, dann
			set_wuerfel_r(-1);	//alle Segmente zum leuchten bringen
			if (count_r == 6) { //von 1-6 zhlen
				count_r = 1;
				} else {
				count_r++;
			}
		}
				
		if (key_r_abbild != key_r_last && key_r_abbild == true) { //steigende Flanke -> losgelassen, da KEY_G invertiert
			key_r_pressed = true;
		}
				
		if (key_r_pressed == true) {										//wenn taste gedrckt, aber inzwischen losgelassen wurde, dann
			if(prescaler_pressed_led_r < prescaler_pressed_led_grenze_r) {	//prescaler - zhlen von 0 bis zur dynamischen prescalerobergrenze (das dynamische wird fr das auslaufen bentigt)
				prescaler_pressed_led_r++;
				} else {
				prescaler_pressed_led_r = 0;
			}
			if (prescaler_grenze_verschieben_r < auslaufen_prescaler) { //prescaler fr die geschwindigkeit des auslaufens -> ruft bei 0 die erhhung des dynamischen prescalers auf
				prescaler_grenze_verschieben_r++;
				} else {
				prescaler_grenze_verschieben_r = 0;
			}
					
					
					
					
			if (prescaler_pressed_led_r == 0) { //wenn overflow des dynamischen prescalers, dann
				if (count_r == 6) { //von 0 bis 6 zhlen
					count_r = 1;
					} else {
					count_r++;
				}
				set_wuerfel_r(count_r);	//und ausgeben
			}
					
			if (prescaler_grenze_verschieben_r == 0) { //wenn overflow des prescalers fr das verschieben der geschwindigkeit, dann
				if (prescaler_pressed_led_grenze_r < max_prescaler_grenze_verschieben) { //wenn langsamste geschwindigkeit noch nicht erreicht, dann
					prescaler_pressed_led_grenze_r++; //geschwindigkeit verlangsamen (durch erhhung des prescalers)
					} else {
					key_r_pressed = false; //sonst alles auf standartwerte setzen und das auslaufen verlassen (zahl bleib stehen!)
					prescaler_pressed_led_grenze_r = min_prescaler_grenze_verschieben;
					prescaler_pressed_led_r = 0;
					prescaler_grenze_verschieben_r = 0;
				}
			}
		}
				
		key_r_last = key_r_abbild;
}